@@ -5,8 +5,8 @@ import pyqtgraph as pg
 from typing import Dict, List, Optional, NamedTuple
 from itertools import zip_longest
 
-from AnyQt.QtGui import QBrush, QColor, QPainterPath
-from AnyQt.QtCore import Qt, QSize
+from AnyQt.QtGui import QBrush, QColor, QPainterPath, QPalette
+from AnyQt.QtCore import Qt, QSize, QEvent
 from AnyQt.QtCore import pyqtSignal as Signal
 from pyqtgraph.functions import mkPen
 from pyqtgraph.graphicsItems.ViewBox import ViewBox
@@ -15,9 +15,9 @@ from lifelines import KaplanMeierFitter
 from lifelines.utils import median_survival_times, format_p_value
 from lifelines.statistics import multivariate_logrank_test
 
-from Orange.data import Table, Domain, DiscreteVariable, ContinuousVariable, TimeVariable
+from Orange.data import Table, DiscreteVariable, Domain
 from Orange.widgets import gui
-from Orange.widgets.widget import Input, Output, OWWidget, Msg
+from Orange.widgets.widget import Input, Output, OWWidget
 from Orange.widgets.settings import (
     Setting,
     ContextSetting,
@@ -26,12 +26,21 @@ from Orange.widgets.settings import (
 )
 from Orange.widgets.utils.plot import SELECT, PANNING, ZOOMING, OWPlotGUI
 from Orange.widgets.utils.itemmodels import DomainModel
+from Orange.widgets.utils.annotated_data import (
+    create_annotated_table,
+    ANNOTATED_DATA_SIGNAL_NAME,
+)
 from Orange.widgets.visualize.owscatterplotgraph import LegendItem
 from Orange.data.filter import IsDefined
 
 
-MEDIAN_LINE_PEN = pg.mkPen(color=QColor(Qt.darkGray), width=1, style=Qt.DashLine)
-HORIZONTAL_LINE = pg.InfiniteLine(pos=0.5, angle=0, pen=MEDIAN_LINE_PEN)
+from orangecontrib.survival_analysis.widgets.data import (
+    check_survival_data,
+    get_survival_endpoints,
+)
+
+
+MEDIAN_LINE_PEN_STYLE = {'color': QColor(Qt.darkGray), 'width': 1, 'style': Qt.DashLine}
 
 
 def create_line_symbol():
@@ -44,12 +53,23 @@ def create_line_symbol():
 class EstimatedFunctionCurve:
     @staticmethod
     def generate_curve_coordinates(timeline, probabilities):
-        intervals = zip_longest(zip(timeline, probabilities), zip(timeline[1:], probabilities[:-1]))
-        x, y = zip(*[coordinate for start, finish in intervals for coordinate in (start, finish) if coordinate])
+        intervals = zip_longest(
+            zip(timeline, probabilities), zip(timeline[1:], probabilities[:-1])
+        )
+        x, y = zip(
+            *[
+                coordinate
+                for start, finish in intervals
+                for coordinate in (start, finish)
+                if coordinate
+            ]
+        )
         return np.array(x), np.array(y)
 
     def __init__(self, time, events, label=None, color=None):
-        self._kmf = KaplanMeierFitter().fit(time.astype(np.float64), events.astype(np.float64))
+        self._kmf = KaplanMeierFitter().fit(
+            time.astype(np.float64), events.astype(np.float64)
+        )
 
         self._label: str = label
         self.color: List[int] = color
@@ -78,7 +98,9 @@ class EstimatedFunctionCurve:
         self.median_survival = median = np.round(
             median_survival_times(self._kmf.survival_function_.astype(np.float32)), 1
         )
-        self.median_vertical = pg.PlotDataItem(x=(median, median), y=(0, 0.5), pen=MEDIAN_LINE_PEN)
+        self.median_vertical = pg.PlotDataItem(
+            x=(median, median), y=(0, 0.5), pen=pg.mkPen(**MEDIAN_LINE_PEN_STYLE)
+        )
 
         censored_data = self.get_censored_data()
 
@@ -103,7 +125,9 @@ class EstimatedFunctionCurve:
         time_events = np.column_stack((self._kmf.durations, self._kmf.event_observed))
         censored_time = time_events[np.argwhere(time_events[:, 1] == 0), 0]
         survival = self._kmf.survival_function_.values
-        return np.column_stack((censored_time, survival[np.where(censored_time == self._kmf.timeline)[1]]))
+        return np.column_stack(
+            (censored_time, survival[np.where(censored_time == self._kmf.timeline)[1]])
+        )
 
     def get_color(self, alpha=255) -> QColor:
         color = QColor(*self.color) if self.color else QColor(Qt.darkGray)
@@ -206,7 +230,9 @@ class CustomLegendItem(LegendItem):
 
     def set_curve(self, curve: EstimatedFunctionCurve):
         curve_label = LabelItem(curve.label, color=curve.get_color())
-        samples = LabelItem(f'{curve.num_of_samples - curve.num_of_censored_samples}/{curve.num_of_samples}')
+        samples = LabelItem(
+            f'{curve.num_of_samples - curve.num_of_censored_samples}/{curve.num_of_samples}'
+        )
         median = LabelItem(f'{curve.median_survival}')
 
         row = self.layout.rowCount()
@@ -239,6 +265,14 @@ class CustomLegendItem(LegendItem):
         self.items = []
         self.updateSize()
 
+    def changeEvent(self, event: QEvent):
+        if event.type() == QEvent.PaletteChange:
+            color = self.palette().color(QPalette.Text)
+            for item in self.items:
+                if isinstance(item, LabelItem):
+                    item.setText(item.text, color=color)
+        super(LegendItem, self).changeEvent(event)
+
 
 class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
     HIGHLIGHT_RADIUS = 20  # in pixels
@@ -258,15 +292,19 @@ class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
         self.view_box: KaplanMeierViewBox = self.getViewBox()
 
         self._mouse_moved_signal = pg.SignalProxy(
-            self.plotItem.scene().sigMouseMoved, slot=self.mouseMovedEvent, delay=0.15, rateLimit=10
+            self.plotItem.scene().sigMouseMoved,
+            slot=self.mouseMovedEvent,
+            delay=0.15,
+            rateLimit=10,
         )
         self.view_box.selection_changed.connect(self.on_selection_changed)
 
         self.legend = CustomLegendItem()
         self.legend.setParentItem(self.getViewBox())
         self.legend.restoreAnchor(((1, 0), (1, 0)))
+        self.legend.hide()
 
-        self.setLabels(left='Survival Probability', bottom=self.parent.time_var_label)
+        self.setLabels(left='Survival Probability', bottom=self.parent.time_var_name)
 
     def mouseMovedEvent(self, ev):
         pos = self.view_box.mapSceneToView(ev[0])
@@ -329,7 +367,9 @@ class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
         """If curve id is None clear all else clear only highlighted curve"""
         if curve_id is not None:
             self.curves[curve_id].selection.hide()
-            self.selection = {key: val for key, val in self.selection.items() if key != curve_id}
+            self.selection = {
+                key: val for key, val in self.selection.items() if key != curve_id
+            }
             return
 
         for curve in self.curves.values():
@@ -371,7 +411,9 @@ class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
         right_selection = (end_x, curve.y[right])
         middle_selection = np.column_stack((curve.x, curve.y))[left:right]
         selected = np.vstack((left_selection, middle_selection, right_selection))
-        self.selection[self.highlighted_curve] = SelectionInterval(selected[:, 0], selected[:, 1])
+        self.selection[self.highlighted_curve] = SelectionInterval(
+            selected[:, 0], selected[:, 1]
+        )
         self.set_selection_item(self.highlighted_curve)
 
         if is_finished:
@@ -385,7 +427,9 @@ class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
             return
 
         if median:
-            self.addItem(HORIZONTAL_LINE)
+            self.addItem(
+                pg.InfiniteLine(pos=0.5, angle=0, pen=pg.mkPen(**MEDIAN_LINE_PEN_STYLE))
+            )
 
         for curve in self.curves.values():
             self.addItem(curve.estimated_fun)
@@ -404,13 +448,13 @@ class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
 
         self.set_selection()
         self.update_legend()
-        self.setLabels(bottom=self.parent.time_var_label)
+        self.setLabels(bottom=self.parent.time_var_name)
 
     def update_legend(self):
         self.legend.hide()
 
         self.legend.set_header()
-        for curve in [curve for curve in self.curves.values()]:
+        for curve in list(self.curves.values()):
             self.legend.set_curve(curve)
         if len(self.curves) > 1:
             self.legend.set_footer(format_p_value(2)(self.parent.log_rank_test.p_value))
@@ -438,9 +482,10 @@ class KaplanMeierPlot(gui.OWComponent, pg.PlotWidget):
 
 class OWKaplanMeier(OWWidget):
     name = 'Kaplan-Meier Plot'
-    description = 'Plot the Kaplan-Meier estimate.'
+    description = 'Visualisation of Kaplan-Meier estimator.'
     icon = 'icons/owkaplanmeier.svg'
-    priority = 0
+    priority = 10
+    keywords = ['Kaplan-Meier', 'survival curve', 'log-rank']
 
     show_confidence_interval: bool
     show_confidence_interval = Setting(False)
@@ -452,44 +497,39 @@ class OWKaplanMeier(OWWidget):
     show_censored_data = Setting(False)
 
     settingsHandler = DomainContextHandler()
-    time_var = ContextSetting(None, schema_only=True)
-    event_var = ContextSetting(None, schema_only=True)
     group_var: Optional[DiscreteVariable] = ContextSetting(None, schema_only=True)
 
     graph = SettingProvider(KaplanMeierPlot)
     graph_name = 'graph.plotItem'
 
-    auto_commit: bool = Setting(False, schema_only=True)
+    auto_commit: bool = Setting(True, schema_only=True)
 
     class Inputs:
         data = Input('Data', Table)
 
     class Outputs:
-        selected_data = Output('Data', Table)
-
-    class Warning(OWWidget.Warning):
-        missing_values = Msg('Rows with missing values were omitted.')
+        selected_data = Output('Selected Data', Table, default=True)
+        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Table)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
         self.data: Optional[Table] = None
-        self._data: Optional[Table] = None
         self.plot_curves = None
         self.log_rank_test = None
 
-        time_var_model = DomainModel(valid_types=(ContinuousVariable,))
-        event_var_model = DomainModel(valid_types=DomainModel.PRIMITIVE)
-        group_var_model = DomainModel(placeholder='(None)', valid_types=(DiscreteVariable,))
-
-        box = gui.vBox(self.controlArea, 'Time', margin=0)
-        gui.comboBox(box, self, 'time_var', model=time_var_model, callback=self.on_controls_changed)
-
-        box = gui.vBox(self.controlArea, 'Event', margin=0)
-        gui.comboBox(box, self, 'event_var', model=event_var_model, callback=self.on_controls_changed)
+        group_var_model = DomainModel(
+            placeholder='(None)', valid_types=(DiscreteVariable,)
+        )
 
         box = gui.vBox(self.controlArea, 'Group', margin=0)
-        gui.comboBox(box, self, 'group_var', model=group_var_model, callback=self.on_controls_changed)
+        gui.comboBox(
+            box,
+            self,
+            'group_var',
+            model=group_var_model,
+            callback=self.on_group_changed,
+        )
 
         box = gui.vBox(self.controlArea, 'Display options')
         gui.checkBox(
@@ -517,7 +557,7 @@ class OWKaplanMeier(OWWidget):
         )
 
         self.graph: KaplanMeierPlot = KaplanMeierPlot(parent=self)
-        self.graph.selection_changed.connect(self.commit)
+        self.graph.selection_changed.connect(self.commit.deferred)
         self.mainArea.layout().addWidget(self.graph)
 
         plot_gui = OWPlotGUI(self)
@@ -525,47 +565,68 @@ class OWKaplanMeier(OWWidget):
 
         gui.rubber(self.controlArea)
 
-        self.commit_button = gui.auto_commit(self.controlArea, self, 'auto_commit', '&Commit', box=False)
-
-    @Inputs.data
-    def set_data(self, data: Table):
-        self.closeContext()
-        if not data:
-            return
-
-        def _filter_domain_model_options(_domain):
-            vars_ = [var for var in _domain.variables + _domain.metas if not isinstance(var, TimeVariable)]
-            vars_ = [
-                var
-                for var in vars_
-                if isinstance(var, ContinuousVariable) or isinstance(var, DiscreteVariable) and len(var.values) <= 2
-            ]
-
-            return Domain(vars_)
-
-        self.data = data
-        domain = _filter_domain_model_options(data.domain)
-        self.controls.time_var.model().set_domain(domain)
-        self.controls.event_var.model().set_domain(domain)
-        self.controls.group_var.model().set_domain(data.domain)
-
-        self.time_var = None
-        self.event_var = None
-        self.group_var = None
-        self.graph.selection = {}
-        self.openContext(data.domain)
-
-        self.graph.curves = {curve_id: curve for curve_id, curve in enumerate(self.generate_plot_curves())}
-        self.graph.update_plot(**self._get_plot_options())
-        self.commit()
+        self.commit_button = gui.auto_commit(
+            self.controlArea, self, 'auto_commit', '&Commit', box=False
+        )
 
     @property
-    def time_var_label(self):
+    def time_var(self):
+        if not self.data:
+            return
+
+        time_var, _ = get_survival_endpoints(self.data.domain)
+        return time_var
+
+    @property
+    def event_var(self):
+        if not self.data:
+            return
+
+        _, event_var = get_survival_endpoints(self.data.domain)
+        return event_var
+
+    @property
+    def time_var_name(self):
         if not self.time_var:
             return 'Time'
 
         return self.time_var.name
 
+    @Inputs.data
+    @check_survival_data
+    def set_data(self, data: Table):
+        self.closeContext()
+
+        if data and data.has_missing():
+            filter_ = IsDefined(columns=data.domain.class_vars)
+            self.data = filter_(data)
+        else:
+            self.data = data
+
+        # exclude class vars
+        domain = data.domain if data else None
+        if domain is not None:
+            domain = Domain(domain.attributes, metas=domain.metas)
+
+        self.controls.group_var.model().set_domain(domain)
+        if domain is not None:
+            group_var_model = self.controls.group_var.model()
+            if 'Cohorts' in domain and domain['Cohorts'] in group_var_model:
+                self.group_var = domain['Cohorts']
+            else:
+                self.group_var = None
+
+        self.graph.selection = {}
+        self.openContext(domain)
+
+        self.graph.curves = {
+            curve_id: curve
+            for curve_id, curve in enumerate(self.generate_plot_curves())
+        }
+        self.graph.update_plot(**self._get_plot_options())
+
+        self.commit.now()
+
     def _get_plot_options(self):
         return {
             'confidence_interval': self.show_confidence_interval,
@@ -576,48 +637,41 @@ class OWKaplanMeier(OWWidget):
     def on_display_option_changed(self) -> None:
         self.graph.update_plot(**self._get_plot_options())
 
-    def on_controls_changed(self):
+    def on_group_changed(self):
         if not self.data:
             return
 
-        self.graph.curves = {curve_id: curve for curve_id, curve in enumerate(self.generate_plot_curves())}
+        self.graph.curves = {
+            curve_id: curve
+            for curve_id, curve in enumerate(self.generate_plot_curves())
+        }
         self.graph.clear_selection()
         self.graph.update_plot(**self._get_plot_options())
-        self.commit()
+        self.commit.now()
 
     def _get_discrete_var_color(self, index: Optional[int]):
         if self.group_var is not None and index is not None:
             return list(self.group_var.colors[index])
 
     def generate_plot_curves(self) -> List[EstimatedFunctionCurve]:
-        self._data = None
-        self.Warning.missing_values.clear()
-
         if self.time_var is None or self.event_var is None:
             return []
 
-        filter_ = IsDefined(columns=[self.time_var, self.event_var])
-
-        # if undefined values are detected in time_var or event_var omit rows with undefined values
-        self._data = filter_(self.data)
-        if len(self.data) == len(self._data):
-            self._data = None
-        else:
-            self.Warning.missing_values()
-
-        data = self.data if self._data is None else self._data
-        time, _ = data.get_column_view(self.time_var)
-        events, _ = data.get_column_view(self.event_var)
+        data = self.data
+        time = data.get_column(self.time_var)
+        events = data.get_column(self.event_var)
 
         if self.group_var:
-            groups, _ = data.get_column_view(self.group_var)
+            groups = data.get_column(self.group_var.name)
             group_indexes = [index for index, _ in enumerate(self.group_var.values)]
             colors = [self._get_discrete_var_color(index) for index in group_indexes]
             masks = groups == np.reshape(group_indexes, (-1, 1))
             self.log_rank_test = multivariate_logrank_test(time, groups, events)
 
             return [
-                EstimatedFunctionCurve(time[mask], events[mask], color=color, label=label)
+                EstimatedFunctionCurve(
+                    time[mask], events[mask], color=color, label=label
+                )
                 for mask, color, label in zip(masks, colors, self.group_var.values)
                 if mask.any()
             ]
@@ -625,29 +679,37 @@ class OWKaplanMeier(OWWidget):
         else:
             return [EstimatedFunctionCurve(time, events)]
 
+    @gui.deferred
     def commit(self):
         if not self.graph.selection:
             self.Outputs.selected_data.send(None)
+            self.Outputs.annotated_data.send(None)
             return
 
-        data = self.data if self._data is None else self._data
+        data = self.data
 
-        time, _ = data.get_column_view(self.time_var)
+        time = data.get_column(self.time_var)
         if self.group_var is None:
             time_interval = self.graph.selection[0].x
             start, end = time_interval[0], time_interval[-1]
-            selection = np.argwhere((time >= start) & (time <= end)).reshape(-1).astype(int)
+            selection = (
+                np.argwhere((time >= start) & (time <= end)).reshape(-1).astype(int)
+            )
         else:
             selection = []
-            group, _ = data.get_column_view(self.group_var)
+            group = data.get_column(self.group_var.name)
             for group_id, time_interval in self.graph.selection.items():
                 start, end = time_interval.x[0], time_interval.x[-1]
                 selection += (
-                    np.argwhere((time >= start) & (time <= end) & (group == group_id)).reshape(-1).astype(int).tolist()
+                    np.argwhere((time >= start) & (time <= end) & (group == group_id))
+                    .reshape(-1)
+                    .astype(int)
+                    .tolist()
                 )
             selection = sorted(selection)
 
         self.Outputs.selected_data.send(data[selection, :])
+        self.Outputs.annotated_data.send(create_annotated_table(data, selection))
 
     def send_report(self):
         if self.data is None:
@@ -661,4 +723,5 @@ class OWKaplanMeier(OWWidget):
 if __name__ == "__main__":
     from orangewidget.utils.widgetpreview import WidgetPreview
 
-    WidgetPreview(OWKaplanMeier).run(Table('iris'))
+    table = Table('http://datasets.biolab.si/core/melanoma.tab')
+    WidgetPreview(OWKaplanMeier).run(input_data=table)
